@@ -26,15 +26,21 @@ module ActiveLdap
             :host => host,
             :port => port,
           }
-          config[:encryption] = {:method => method} if method
+          if method
+            config[:encryption] = { :method => method }
+            config[:encryption][:tls_options] = @tls_options if @tls_options
+          end
           begin
             uri = construct_uri(host, port, method == :simple_tls)
             with_start_tls = method == :start_tls
             info = {:uri => uri, :with_start_tls => with_start_tls}
             [log("connect", info) {Net::LDAP::Connection.new(config)},
              uri, with_start_tls]
-          rescue Net::LDAP::LdapError
-            raise ConnectionError, $!.message
+          rescue Net::LDAP::ConnectionError => error
+            raise ConnectionError, error.message
+          rescue Net::LDAP::Error => error
+            message = "#{error.class}: #{error.message}"
+            raise ConnectionError, message, caller(0) + error.backtrace
           end
         end
       end
@@ -50,7 +56,7 @@ module ActiveLdap
       def bind(options={})
         begin
           super
-        rescue Net::LDAP::LdapError
+        rescue Net::LDAP::Error
           raise AuthenticationError, $!.message
         end
       end
@@ -148,8 +154,13 @@ module ActiveLdap
         result = log(name, info) do
           begin
             @connection.send(method, *args, &block)
-          rescue Errno::EPIPE, Errno::ECONNRESET
-            raise ConnectionError, "#{$!.class}: #{$!.message}"
+          rescue SystemCallError => error
+            message = "#{error.class}: #{error.message}"
+            raise ConnectionError, message, caller(0) + error.backtrace
+          rescue Net::LDAP::ResponseMissingOrInvalidError => error
+            message = "#{error.class}: #{error.message}"
+            message << ": connection may be timed out"
+            raise ConnectionError, message, caller(0) + error.backtrace
           end
         end
         message = nil
